/*
  $Id: uim-custom.c,v 1.2 2008/04/28 12:56:57 masahino Exp $
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <stdarg.h>
#include <signal.h>
#include <sys/time.h>
#include <time.h>

#include <locale.h>

#include <uim/uim.h>
#include <uim/uim-custom.h>

#include <ruby.h>

extern VALUE rb_UIM;
VALUE rb_UIM_Custom;
VALUE rb_UIM_Custom_Group;

static VALUE rb_uim_custom_get(VALUE self, VALUE custom_sym)
{
     struct uim_custom *c;
     VALUE obj;
     Check_Type(custom_sym, T_STRING);

     c = uim_custom_get(StringValuePtr(custom_sym));
     obj = Data_Wrap_Struct(self, NULL, -1, c);
     return obj;
}

static VALUE rb_uim_custom_enable(VALUE self)
{
     uim_bool ret;
     ret = uim_custom_enable();
     if (ret == UIM_TRUE) 
	  return Qtrue;
     else
	  return Qfalse;
}

/* load & save */
static VALUE rb_uim_custom_load(VALUE self)
{
     uim_bool ret;

     ret = uim_custom_load();
     if (ret == UIM_TRUE) {
	  return Qtrue;
     } else {
	  return Qfalse;
     }
}

static VALUE rb_uim_custom_save(VALUE self)
{
     uim_bool ret;

     ret = uim_custom_save();
     if (ret == UIM_TRUE) {
	  return Qtrue;
     } else {
	  return Qfalse;
     }
}


/* accessor */
static VALUE rb_uim_custom_get_label(VALUE self)
{
     struct uim_custom *custom;
     Data_Get_Struct(self, struct uim_custom, custom);
     return rb_str_new2(custom->label);
}

static VALUE rb_uim_custom_get_desc(VALUE self)
{
     struct uim_custom *custom;
     Data_Get_Struct(self, struct uim_custom, custom);
     return rb_str_new2(custom->desc);
}

/*
 * literalization 
 */
static VALUE rb_uim_custom_definition_as_literal(VALUE self, VALUE custom_sym)
{
     Check_Type(custom_sym, T_STRING);
     return rb_str_new2(uim_custom_definition_as_literal(StringValuePtr(custom_sym)));
}

/*
 * custom group *
 */
static VALUE rb_uim_custom_group_get(VALUE self, VALUE group_sym)
{
     struct uim_custom_group *group;
     VALUE obj;
     
     Check_Type(group_sym, T_STRING);
     group = uim_custom_group_get(StringValuePtr(group_sym));
     obj = Data_Wrap_Struct(self, NULL, -1, group);
     return obj;
}

/*
 * custom symbol list
 */
static VALUE rb_uim_custom_collect_by_group(VALUE self, VALUE group_sym)
{
     char **custom_syms, **custom_sym;
     VALUE rb_custom_syms;

     Check_Type(group_sym, T_STRING);
     
     custom_syms = uim_custom_collect_by_group(StringValuePtr(group_sym));
     rb_custom_syms = rb_ary_new();
     if (custom_syms) {
	  for (custom_sym = custom_syms; *custom_sym; custom_sym++) {
	       rb_ary_push(rb_custom_syms, rb_str_new2(*custom_sym));
	  }
     }
     uim_custom_symbol_list_free(custom_syms);
     return rb_custom_syms;
}

static VALUE rb_uim_custom_group_get_label(VALUE self)
{
     struct uim_custom_group *group;
     Data_Get_Struct(self, struct uim_custom_group, group);
     return rb_str_new2(group->label);
}

/*
 * group symbol list 
 */

static VALUE rb_uim_custom_primary_groups(VALUE self)
{
     char **primary_groups, **grp;
     VALUE rb_groups;
     primary_groups = uim_custom_primary_groups();
     rb_groups = rb_ary_new();
     for (grp = primary_groups; *grp; grp++) {
	  rb_ary_push(rb_groups, rb_str_new2(*grp));
     }
     uim_custom_symbol_list_free(primary_groups);
     return rb_groups;
}

void Init_uim_custom(void) 
{

	/* UIM::Custom */
	rb_UIM_Custom = rb_define_class_under(rb_UIM, "Custom", rb_cObject);
	rb_define_singleton_method(rb_UIM_Custom, "new",
				   rb_uim_custom_get, 1);
	rb_define_singleton_method(rb_UIM_Custom, "enable",
				   rb_uim_custom_enable, 0);
	/* load & save */
	rb_define_singleton_method(rb_UIM_Custom, "load",
				   rb_uim_custom_load, 0);
	rb_define_singleton_method(rb_UIM_Custom, "save",
				   rb_uim_custom_save, 0);
	
	rb_define_singleton_method(rb_UIM_Custom, "primary_groups",
				   rb_uim_custom_primary_groups, 0);
	rb_define_singleton_method(rb_UIM_Custom, "collect_by_group",
				   rb_uim_custom_collect_by_group, 1);

	/* accessor */
	rb_define_method(rb_UIM_Custom, "label",
			 rb_uim_custom_get_label, 0);
	rb_define_method(rb_UIM_Custom, "desc",
			 rb_uim_custom_get_desc, 0);

	/* literalization */
	rb_define_singleton_method(rb_UIM_Custom, "definition_as_literal",
				  rb_uim_custom_definition_as_literal, 1);

	/* UIM::Custom::Group */
	rb_UIM_Custom_Group = rb_define_class_under(rb_UIM_Custom, "Group", 
						    rb_cObject);

	rb_define_singleton_method(rb_UIM_Custom_Group, "new",
				   rb_uim_custom_group_get, 1);

	/* accessor */
	rb_define_method(rb_UIM_Custom_Group, "label",
			 rb_uim_custom_group_get_label, 0);
	
}
